package com.health.actiondemo

import android.app.Activity
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.Log
import android.widget.TextView
import com.alibaba.fastjson.JSON
import com.health.baselibrary.utils.StringUtils
import com.health.videoplay.bean.StandardConditonsModel
import com.health.videoplay.util.CalFrameDelegate
import com.healthhope.opengl.PoseCameraView
import java.io.BufferedReader
import java.io.InputStreamReader
import java.lang.StringBuilder
import java.util.HashMap

class ActionCountActivity : Activity() {

    private var poseView:PoseCameraView? = null
    private var textInfo: TextView? = null
    private var textCount: TextView? = null
    private var textResult: TextView? = null

    private var textDebug1:TextView? = null
    private var textDebug2:TextView? = null
    private var textDebug3:TextView? = null

    private val calFrameDelegate = CalFrameDelegate()
    private val mainHandler = Handler(Looper.getMainLooper())
    private var containJump = false
    private var aCount = 0

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_action_count)
        initInfo()
        textInfo = findViewById(R.id.text_info)
        textCount = findViewById(R.id.text_count)
        textResult = findViewById(R.id.result_info)

        textDebug1 = findViewById(R.id.text_debug_1)
        textDebug2 = findViewById(R.id.text_debug_2)
        textDebug3 = findViewById(R.id.text_debug_3)

        val intExtra = intent.getIntExtra("actionType", 0)

        //动作检查
        calFrameDelegate.setCheckType(CalFrameDelegate.TYPE_ACTION_CHECK)
        when(intExtra) {
            //蹲起
            0 -> {
                textInfo?.text = "蹲起："
                val info:List<StandardConditonsModel> = JSON.parseArray(getRawText(R.raw.action_dunqi), StandardConditonsModel::class.java)
                calFrameDelegate.setActionCheckConditions(info)
            }
            //开合跳
            1 -> {
                textInfo?.text = "开合跳："
                containJump = true
                val info:List<StandardConditonsModel> = JSON.parseArray(getRawText(R.raw.gameinfo), StandardConditonsModel::class.java)
                calFrameDelegate.setActionCheckConditions(info)
            }
            //跳
            2 -> {
                textInfo?.text = "跳跃："

            }
        }
        calFrameDelegate.setGainResultListener(object: CalFrameDelegate.OnGainResultListener{
            override fun onResult(conditonsModel: StandardConditonsModel?, result: Byte) {
                Log.i("ademo","result--->" + result)
                setResult(result.toString())
                when (result) {
                    CalFrameDelegate.ResultCode.RESULT_OK -> {
                        if (containJump) {
//                            if (poseView?.getRenderHelper()?.isJump == true) {
                                updateInfo()
//                            }
                        } else {
                            updateInfo()
                        }

                    }
                }
            }

            override fun onWakeUpProcess(wakeUpProcess: Float) {

            }
        })
        calFrameDelegate.setInfoListener(object: CalFrameDelegate.OnInfoListener{
            override fun onConstantInfo(constants: MutableMap<String, Double>?) {
                val sb = StringBuilder()
                for ((key, value) in constants!!) {
                    sb.append(transConstantsNameToCh(key) + ":" + StringUtils.formatNum(value, 1))
                        .append("\n")
                }
                mainHandler.post{
                    textDebug1?.text = sb.toString()

                }
            }

            override fun onCondition(conditionType: String?, bodyKeys: MutableList<String>?, ax: Double, a: Double, b: Double, by: Double) {
                if (TextUtils.isEmpty(conditionType)) {
                    runOnUiThread { textDebug2?.setText("无标注信息") }
                    return
                }
                val sb = StringBuilder()

                if ("0" == conditionType) {
                    sb.append("角度").append("\n")
                } else if ("1" == conditionType) {
                    sb.append("高差").append("\n")
                } else if ("2" == conditionType) {
                    sb.append("水平差").append("\n")
                } else {
                    sb.append("距离").append("\n")
                }
//            sb.append("conditionType:" + conditionType).append("\n");
                //            sb.append("conditionType:" + conditionType).append("\n");
                sb.append("calPoints:")
                for (i in bodyKeys!!.indices) {
                    if (i < bodyKeys.size - 1) {
                        sb.append(pointChNames.get(bodyKeys[i])).append("_")
                    } else {
                        sb.append(pointChNames.get(bodyKeys[i]))
                    }
                }
                sb.append("\n")
                sb.append("ax=").append(StringUtils.formatNum(ax, 1)).append("\n")
                    .append("a=").append(StringUtils.formatNum(a, 1)).append("\n")
                    .append("b=").append(StringUtils.formatNum(b, 1)).append("\n")
                    .append("by=").append(StringUtils.formatNum(by, 1))
                runOnUiThread { textDebug2?.setText(sb.toString()) }
            }

            override fun onCalInfo(calResult: Double) {
                var cal = calResult
                if (java.lang.Double.isNaN(cal) || java.lang.Double.isInfinite(cal)) {
                    cal = 0.0
                }
                runOnUiThread {
                    textDebug3?.setText(
                        """
                        calResult:
                        ${StringUtils.formatNum(cal, 1)}
                        """.trimIndent()
                    )
                }
            }

        })
        poseView = findViewById(R.id.pose_camera)
        poseView?.getRenderHelper()?.setOnFrameListener { pointFS ->
            Log.i("ademo","on onFrame")
            calFrameDelegate.onFrame(pointFS)
        }
    }

    override fun onResume() {
        super.onResume()
        poseView?.onResume()
    }

    override fun onPause() {
        super.onPause()
        poseView?.onPause()
    }

    private fun updateInfo() {
        mainHandler.post{
            aCount++
            textCount?.text = aCount.toString()
        }
    }

    private fun setResult(info: String) {
        mainHandler.post{
            textResult?.text = "result-->$info"
        }
    }

    private fun getRawText(resId: Int):String {
        val openRawResource = resources.openRawResource(resId)
        val bufferedReader = BufferedReader(InputStreamReader(openRawResource))
        return bufferedReader.readText()
    }

//    private val bodyConstants = HashMap<String, Double>()
    private val pointChNames = HashMap<String, String>()

    fun initInfo() {
//        /** 头顶-脖子 高差 */
//        bodyConstants["OH_NK_1"] = -65.36
//        /** 左膝-左踝 高差*/
//        bodyConstants["LK_LA_1"] = -76.64
//        /** 右膝-右踝 高差 */
//        bodyConstants["RK_RA_1"] = -69.72
//        /** 左胯-左膝 高差 */
//        bodyConstants["LP_LK_1"] = -75.49
//        /** 右胯-右膝 高差 */
//        bodyConstants["RP_RK_1"] = -82.85
//        /** 左肩-右肩 水平*/
//        bodyConstants["LS_RS_2"] = -80.90
//        /** 左肩-右肩 连线*/
//        bodyConstants["LS_RS_3"] = 80.91
        pointChNames["RK"] = "右膝"
        pointChNames["LK"] = "左膝"
        pointChNames["RA"] = "右踝"
        pointChNames["LA"] = "左踝"
        pointChNames["LW"] = "左腕"
        pointChNames["RW"] = "右腕"
        pointChNames["LE"] = "左耳"
        pointChNames["RE"] = "右耳"
        pointChNames["LS"] = "左肩"
        pointChNames["RS"] = "右肩"
        pointChNames["LL"] = "左水平"
        pointChNames["RL"] = "右水平"
        pointChNames["LP"] = "左髋"
        pointChNames["RP"] = "右髋"
        pointChNames["LEW"] = "左肘"
        pointChNames["REW"] = "右肘"
        pointChNames["OH"] = "头顶"
        pointChNames["NK"] = "脖子"
    }

    private fun transConstantsNameToCh(key: String): String? {
        val sb = StringBuilder()
        val s = key.split("_").toTypedArray()
        sb.append(pointChNames[s[0]]).append("_")
        sb.append(pointChNames[s[1]]).append("_")
        if ("0" == s[2]) {
            sb.append("角度")
        } else if ("1" == s[2]) {
            sb.append("高差")
        } else if ("2" == s[2]) {
            sb.append("水平差")
        } else {
            sb.append("距离")
        }
        return sb.toString()
    }
}